import isFunction from '@roots/bud-support/isFunction';
/**
 * Gracefully shutdown {@link Bud} and registered {@link Services}
 *
 * @example
 * ```js
 * bud.close()
 * ```
 */
export function close(onComplete) {
    try {
        if (this.compiler?.instance?.running) {
            this.compiler.instance.close(() => {
                closeDevelopmentServer(this);
            });
        }
        else {
            closeDevelopmentServer(this);
        }
    }
    catch (error) {
        throw error;
    }
    if (onComplete)
        return onComplete();
}
const closeDevelopmentServer = (bud) => {
    if (!bud.server)
        return;
    try {
        if (bud.isDevelopment &&
            isFunction(bud.server.watcher?.instance?.close)) {
            bud.server.watcher.instance.close();
        }
        if (bud.isDevelopment &&
            isFunction(bud.server.connection?.instance?.close)) {
            bud.server.connection.instance.close();
        }
    }
    catch (error) {
        throw error;
    }
};
